;;;; fp-utils-test  -*- Scheme -*-
;;;; Kon Lovett, Sep '18

(import test)

(test-begin "Fp Utils")

;;;

(import (chicken base))
(import  (chicken flonum))

(import fp-inlines fp-utils)

(test-group "Fp Inlines"

  (test-assert (fpzero? 0.0))
  (test-assert (not (fpzero? 1.0)))
  (test-assert (not (fpzero? maximum-flonum)))
  (test-assert (not (fpzero? minimum-flonum)))

  (test-assert (not (fppositive? 0.0)))
  (test-assert (not (fppositive? (fpneg minimum-flonum))))
  (test-assert (fppositive? maximum-flonum))

  (test-assert (fpnatural? 0.0))
  (test-assert (not (fpnatural? (fpneg minimum-flonum))))
  (test-assert (fpnatural? maximum-flonum))

  (test-assert (not (fpnegative? 0.0)))
  (test-assert (fpnegative? (fpneg minimum-flonum)))
  (test-assert (not (fpnegative? maximum-flonum)))

	(test-assert (not (fpeven? 7.0)))
	(test-assert (fpeven? 6.0))
	(test-assert (not (fpodd? 6.0)))
	(test-assert (fpodd? 7.0))

	(test 4.0 (fpadd1 3.0))
	(test 2.0 (fpsub1 3.0))

	(test 27.0 (fpcub 3.0))
)

;;

(define-constant 5eps (fp/ 9.0 1e06))

(test-group "Fp Approximate ="

  ;sweet, sweet repeating expansion
  (let ((x (fp+ 0.15 0.15)) (y (fp+ 0.1 0.2)))
	  (test-assert ".15+.15 <> .1+.2" (not (fp= x y)))
	  (test-assert ".15+.15 ~= .1+.2" (fp~= x y)) )

  ;own epsilon
	(test-assert (fp~= 0.123456 0.123457 5eps))
	(test-assert (fp~<= 0.123456 0.123457 5eps))
	(test-assert (fp~>= 0.123456 0.123457 5eps))
	(test-assert (fp~<= 0.123456 0.12346 5eps))
	(test-assert (fp~>= 0.123456 0.12344 5eps))
)

;;

(define-constant 4eps (fp/ 9.0 1e05))

(test-group "Fp Utils"

	(test 1.0 (fpmodulo 5.0 2.0))
	(test 0.0 (fpmodulo 0.0 1.0))

	(test 2.0 (fpquotient 5.0 2.0))
	(test 1.0 (fpremainder 5.0 2.0))

	(test-assert (flonum? (fprandom)))
	(test-assert (flonum? (fprandom 2456)))

  (parameterize ((current-test-epsilon 4eps))
	  (test 5.6568 (fpdistance 1.0 1.0 5.0 5.0)) )

	(receive (mx mn) (fpmax-and-min 1.0 -1.0 -16.0 13.0 2.0 16.0 7.0 -8.0)
	  (test "fpmax-and-min max" 16.0 mx)
	  (test "fpmax-and-min min" -16.0 mn) )
)

#;
(begin
  (import (chicken flonum) fp-inlines fp-utils)
  (do ((i 0 (add1 i)) )
      ((>= i 100000) )
    (let ((n (fprandom 0.05 0.002)))
      (print n)
      (assert (<= 0.002 n))
      (assert (<= n 0.05)) ) ) )

;;;

(test-end "Fp Utils")

(test-exit)
